import pytest
import responses

from adhan_pi.utils import get_location_from_query


@pytest.fixture(scope="session")
def prayer_api_url():
    return "https://api.aladhan.com/v1/calendar"


@pytest.fixture(autouse=True)
def clear_cache():
    get_location_from_query.cache_clear()


@pytest.fixture()
def prayer_api_200_response(prayer_api_url):
    response = {
        "code": 200,
        "status": "OK",
        "data": [
            {
                "timings": {
                    "Fajr": "04:58 (EST)",
                    "Sunrise": "07:20 (EST)",
                    "Dhuhr": "12:00 (EST)",
                    "Asr": "14:22 (EST)",
                    "Sunset": "16:40 (EST)",
                    "Maghrib": "16:40 (EST)",
                    "Isha": "18:02 (EST)",
                    "Imsak": "05:48 (EST)",
                    "Midnight": "00:00 (EST)",
                },
                "date": {
                    "readable": "01 Jan 2021",
                    "timestamp": "1609509661",
                    "gregorian": {
                        "date": "01-01-2021",
                        "format": "DD-MM-YYYY",
                        "day": "01",
                        "weekday": {"en": "Friday"},
                        "month": {"number": 1, "en": "January"},
                        "year": "2021",
                        "designation": {
                            "abbreviated": "AD",
                            "expanded": "Anno Domini",
                        },
                    },
                },
            },
            {
                "timings": {
                    "Fajr": "04:59 (EST)",
                    "Sunrise": "07:21 (EST)",
                    "Dhuhr": "12:01 (EST)",
                    "Asr": "14:23 (EST)",
                    "Sunset": "16:41 (EST)",
                    "Maghrib": "16:41 (EST)",
                    "Isha": "18:03 (EST)",
                    "Imsak": "05:49 (EST)",
                    "Midnight": "00:00 (EST)",
                },
                "date": {
                    "readable": "02 Jan 2021",
                    "timestamp": "1609509661",
                    "gregorian": {
                        "date": "02-01-2021",
                        "format": "DD-MM-YYYY",
                        "day": "02",
                        "weekday": {"en": "Saturday"},
                        "month": {"number": 1, "en": "January"},
                        "year": "2021",
                        "designation": {
                            "abbreviated": "AD",
                            "expanded": "Anno Domini",
                        },
                    },
                },
            },
        ],
    }
    responses.add(
        responses.GET,
        prayer_api_url,
        json=response,
        status=200,
        match_querystring=False,
    )
    return response


@pytest.fixture()
def prayer_api_200_response_different_version_response(prayer_api_url):
    response = {
        "code": 200,
        "status": "OK",
        "data": [
            {
                "timings": {
                    "The Fajr": "04:58 (EST)",
                    "The Sunrise": "07:20 (EST)",
                    "The Dhuhr": "12:00 (EST)",
                    "The Asr": "14:22 (EST)",
                    "The Sunset": "16:40 (EST)",
                    "The Maghrib": "16:40 (EST)",
                    "The Isha": "18:02 (EST)",
                    "The Imsak": "05:48 (EST)",
                    "The Midnight": "00:00 (EST)",
                },
                "date": {
                    "readable": "01 Jan 2021",
                    "timestamp": "1609509661",
                    "gregorian": {
                        "date": "01-01-2021",
                        "format": "DD-MM-YYYY",
                        "day": "01",
                        "weekday": {"en": "Friday"},
                        "month": {"number": 1, "en": "January"},
                        "year": "2021",
                        "designation": {
                            "abbreviated": "AD",
                            "expanded": "Anno Domini",
                        },
                    },
                },
            },
        ],
    }
    responses.add(
        responses.GET,
        prayer_api_url,
        json=response,
        status=200,
        match_querystring=False,
    )
    return response


@pytest.fixture()
def prayer_api_400_response(prayer_api_url):
    response = {
        "code": 400,
        "status": "BAD",
    }
    responses.add(
        responses.GET,
        prayer_api_url,
        json=response,
        status=400,
        match_querystring=False,
    )



import datetime as dt
import functools
import itertools

import requests
from geopy.geocoders import Nominatim

from .dataclasses import Coordinates, Prayer, PrayerTimes
from .exceptions import LocationNotFoundError, PrayerAPIError


@functools.lru_cache(maxsize=None)
def get_location_from_query(query: str) -> Coordinates:
    geolocator = Nominatim(user_agent="adhan-pi")
    location = geolocator.geocode(query)
    if location is None:
        raise LocationNotFoundError(query)
    return Coordinates(
        latitude=location.latitude, longitude=location.longitude
    )


class PrayertimesAPI(object):
    API_URL = "https://api.aladhan.com/v1/calendar"

    def __init__(self):
        self.session = requests.Session()
        self.session.mount(
            "http://", requests.adapters.HTTPAdapter(max_retries=4)
        )
        self.session.mount(
            "https://", requests.adapters.HTTPAdapter(max_retries=4)
        )

    def get_prayer_times(
        self, location: Coordinates, date: dt.date
    ) -> PrayerTimes:
        response = self.session.get(
            self.API_URL,
            params=dict(
                latitude=location.latitude,
                longitude=location.longitude,
                method="02",
                month=date.strftime("%m"),
                year=date.strftime("%y"),
            ),
            timeout=(0.5, 0.5),
        )
        if response.status_code != 200:
            raise PrayerAPIError(response=response)
        data = response.json()
        try:
            timings = list(
                itertools.dropwhile(
                    lambda d: d["date"]["gregorian"]["date"]
                    != date.strftime("%d-%m-%Y"),
                    data["data"],
                )
            )[0]["timings"]
            return PrayerTimes(
                date=date,
                fajr=Prayer(name="fajr", time=timings["Fajr"]),
                dhuhr=Prayer(name="dhuhr", time=timings["Dhuhr"]),
                asr=Prayer(name="asr", time=timings["Asr"]),
                maghrib=Prayer(name="maghrib", time=timings["Maghrib"]),
                isha=Prayer(name="isha", time=timings["Isha"]),
            )
        except (IndexError, KeyError):
            raise PrayerAPIError(data=data)



import datetime as dt
import pwd
from abc import ABC, abstractmethod
from argparse import ArgumentParser

import adhan_pi

from .config import ADHAN_MP3_PATH, FAJR_ADHAN_MP3_PATH
from .utils import get_location_from_query

try:
    from crontab import CronTab
    from pydub import AudioSegment
    from pydub.playback import play

    CRON_SCRIPTS_IMPORTED = True
except ImportError:
    CRON_SCRIPTS_IMPORTED = False


def schedule_prayer_cron_runner():
    parser = ArgumentParser()
    parser.add_argument("--user", required=True)
    parser.add_argument("--query", required=True)
    args = parser.parse_args()
    schedule_prayer_cron(args.user, args.query)


def schedule_prayer_cron(user: str, query: str):
    if not CRON_SCRIPTS_IMPORTED:
        raise ImportError

    user_id = pwd.getpwnam(user).pw_uid

    prayer_times = adhan_pi.p.get_prayer_times(
        get_location_from_query(query), dt.date.today()
    )

    with CronTab(user=user) as cron:
        for old_job in cron.find_comment("adhan_pi"):
            cron.remove(old_job)

        for prayer in prayer_times:
            job = cron.new(
                command="XDG_RUNTIME_DIR=/run/user/{user_id} /opt/adhan-pi/env/bin/alert_adhan --prayer {prayer} > /dev/null 2>&1".format(
                    user_id=user_id, prayer=prayer.name
                ),
                comment="adhan_pi",
            )
            job.day.every(1)
            job.hour.on(prayer.time.hour)
            job.minute.on(prayer.time.minute)


class AdhanAlert(ABC):
    def __init__(self, prayer: str):
        self.prayer = prayer

    @abstractmethod
    def alert(self):
        pass


class AdhanAlertFFMPEG(AdhanAlert):
    """
    play adhan sound
    """

    def alert(self):
        if not CRON_SCRIPTS_IMPORTED:
            raise ImportError

        if self.prayer == "fajr":
            adhan = AudioSegment.from_mp3(FAJR_ADHAN_MP3_PATH)
        else:
            adhan = AudioSegment.from_mp3(ADHAN_MP3_PATH)
        play(adhan)


def alert_factory(prayer: str):
    return AdhanAlertFFMPEG(prayer)


def alert_adhan():
    parser = ArgumentParser()
    parser.add_argument("--prayer", required=True)
    args = parser.parse_args()
    alert_cls = alert_factory(args.prayer)
    alert_cls.alert()



import datetime as dt

import pytest
import responses
from automock import get_mock

import adhan_pi.utils as utils


def test_get_location():
    location = utils.get_location_from_query("New York, NY")
    assert (0, 1) == tuple(location)
    assert location.latitude == 0
    assert location.longitude == 1
    mock = get_mock("adhan_pi.utils.Nominatim")
    mock.assert_called_with(user_agent="adhan-pi")
    mock().geocode.assert_called_with("New York, NY")


@responses.activate
def test_get_prayer_times(prayer_api_200_response):
    location = utils.get_location_from_query("New York, NY")
    p = utils.PrayertimesAPI()
    day = dt.date(year=2021, month=1, day=1)
    prayer_times = p.get_prayer_times(location, day)
    assert isinstance(prayer_times, utils.PrayerTimes)
    assert prayer_times.fajr.name == "fajr"
    assert prayer_times.fajr.time == dt.time(hour=4, minute=58)
    assert prayer_times.date == day
    assert [
        ("fajr", dt.time(hour=4, minute=58)),
        ("dhuhr", dt.time(hour=12)),
        ("asr", dt.time(hour=14, minute=22)),
        ("maghrib", dt.time(hour=16, minute=40)),
        ("isha", dt.time(hour=18, minute=2)),
    ] == [(p.name, p.time) for p in prayer_times]


@responses.activate
def test_get_prayer_times_200_bad_response(
    prayer_api_200_response_different_version_response,
):
    location = utils.get_location_from_query("New York, NY")
    p = utils.PrayertimesAPI()
    day = dt.date(year=2021, month=1, day=1)
    with pytest.raises(utils.PrayerAPIError):
        p.get_prayer_times(location, day)


@responses.activate
def test_get_prayer_times_400(prayer_api_400_response):
    location = utils.get_location_from_query("New York, NY")
    p = utils.PrayertimesAPI()
    day = dt.date(year=2021, month=1, day=1)
    with pytest.raises(utils.PrayerAPIError):
        p.get_prayer_times(location, day)


@responses.activate
def test_get_prayer_times_correct_day(prayer_api_200_response):
    location = utils.get_location_from_query("New York, NY")
    p = utils.PrayertimesAPI()
    # day marked as the second of january
    day = dt.date(year=2021, month=1, day=2)
    prayer_times = p.get_prayer_times(location, day)
    assert isinstance(prayer_times, utils.PrayerTimes)
    assert prayer_times.fajr.name == "fajr"
    assert prayer_times.fajr.time == dt.time(hour=4, minute=59)



from typing import Dict, Optional

import requests


class PrayerAPIError(Exception):
    def __init__(
        self,
        response: Optional[requests.Response] = None,
        data: Optional[Dict] = None,
    ):
        if response:
            super().__init__(
                "Could not find prayertimes status={status}".format(
                    status=response.status_code
                )
            )
        elif data:
            super().__init__(
                "Could not find prayertimes data={data}".format(data=data)
            )
        else:
            super().__init__()


class LocationNotFoundError(Exception):
    def __init__(self, query: str):
        super().__init__("Could not find location for {}".format(query))



from unittest.mock import MagicMock, Mock

import automock
from geopy.location import Location


def geocode_mock(failed_lookup: bool = False) -> Mock:
    m = Mock()
    if failed_lookup:
        m.return_value.geocode.return_value = None
    else:
        m.return_value.geocode.return_value = Location(
            "nyc", (0, 1), {"address": "nyc", "longitude": 0, "latitude": 0}
        )
    return m


def cron_mock(previous_jobs: bool = False) -> MagicMock:
    m = MagicMock()
    if not previous_jobs:
        m.return_value.__enter__.return_value.find_comment.return_value = []
    else:
        m.return_value.__enter__.return_value.find_comment.return_value = [
            0,
            1,
            2,
            3,
            4,
        ]
    import crontab

    def new_job(command, comment):
        job = crontab.CronItem(
            command=command,
            comment=comment,
            user=None,
            cron=None,
            pre_comment=False,
        )
        return job

    m.return_value.__enter__.return_value.new.side_effect = new_job
    return m


def pwd_mock(valid: bool = True) -> Mock:
    m = Mock()
    if valid:
        m.getpwnam.return_value.pw_uid = 1000
    else:
        m.getpwnam.side_effect = KeyError
    return m


automock.register("adhan_pi.utils.Nominatim", geocode_mock)
automock.register("adhan_pi.cli.CronTab", cron_mock)
automock.register("adhan_pi.cli.AudioSegment")
automock.register("adhan_pi.cli.play")
automock.register("adhan_pi.cli.pwd", pwd_mock)



import datetime as dt
from collections.abc import Iterable, Iterator
from typing import NamedTuple, Union

import attr


def extract_time(t: Union[dt.datetime, str]) -> dt.time:
    if isinstance(t, str):
        t = dt.datetime.strptime(t[:5], "%H:%M")
    return t.time()


@attr.s
class Prayer(object):
    name = attr.ib(type=str)
    time = attr.ib(type=dt.time, converter=extract_time)


class Coordinates(NamedTuple):
    latitude: float
    longitude: float


@attr.s
class PrayerTimes(Iterable):
    date = attr.ib(type=dt.date)
    fajr = attr.ib(type=Prayer)
    dhuhr = attr.ib(type=Prayer)
    asr = attr.ib(type=Prayer)
    maghrib = attr.ib(type=Prayer)
    isha = attr.ib(type=Prayer)

    def __iter__(self):
        return PrayerIterator(self)


class PrayerIterator(Iterator):
    def __init__(self, pt: PrayerTimes):
        self.index = 0
        self.pt = pt

    def __iter__(self):
        return self

    def __next__(self) -> Prayer:
        if self.index == 0:
            prayer = self.pt.fajr
        elif self.index == 1:
            prayer = self.pt.dhuhr
        elif self.index == 2:
            prayer = self.pt.asr
        elif self.index == 3:
            prayer = self.pt.maghrib
        elif self.index == 4:
            prayer = self.pt.isha
        else:
            raise StopIteration
        self.index += 1
        return prayer



import pytest
import responses
from automock import get_mock, swap_mock

import adhan_pi.cli as cli


def test_alert_adhan_fajr():
    cli.AdhanAlertFFMPEG("fajr").alert()
    audio_mock = get_mock("adhan_pi.cli.AudioSegment")
    play_mock = get_mock("adhan_pi.cli.play")
    audio_mock.from_mp3.assert_called_with("fajr.mp3")
    play_mock.assert_called_with(audio_mock.from_mp3())


def test_alert_adhan():
    cli.AdhanAlertFFMPEG("dhuhr").alert()
    audio_mock = get_mock("adhan_pi.cli.AudioSegment")
    play_mock = get_mock("adhan_pi.cli.play")
    audio_mock.from_mp3.assert_called_with("adhan.mp3")
    play_mock.assert_called_with(audio_mock.from_mp3())


@responses.activate
@pytest.mark.freeze_time("2021-01-01")
def test_schedule_no_previous_jobs(prayer_api_200_response):
    cli.schedule_prayer_cron("salah", "Los Angeles")
    pwd_mock = get_mock("adhan_pi.cli.pwd")
    pwd_mock.getpwnam.assert_called_with("salah")
    geopy_mock = get_mock("adhan_pi.utils.Nominatim")
    geopy_mock.assert_called_with(user_agent="adhan-pi")
    geopy_mock().geocode.assert_called_with("Los Angeles")
    cron_mock = get_mock("adhan_pi.cli.CronTab")
    cron_mock.assert_called_with(user="salah")
    cron_mock().__enter__().find_comment.assert_called_with("adhan_pi")
    cron_mock().__enter__().remove.assert_not_called()
    for p in ["fajr", "dhuhr", "asr", "maghrib", "isha"]:
        cron_mock().__enter__().new.assert_any_call(
            command="XDG_RUNTIME_DIR=/run/user/{user_id} /opt/adhan-pi/env/bin/alert_adhan --prayer {prayer} > /dev/null 2>&1".format(
                user_id=1000, prayer=p,
            ),
            comment="adhan_pi",
        )


@responses.activate
@pytest.mark.freeze_time("2021-01-01")
def test_schedule_previous_jobs(prayer_api_200_response):
    with swap_mock("adhan_pi.cli.CronTab", previous_jobs=True):
        cli.schedule_prayer_cron("salah", "Los Angeles")
        cron_mock = get_mock("adhan_pi.cli.CronTab")
        cron_mock.assert_called_with(user="salah")
        cron_mock().__enter__().find_comment.assert_called_with("adhan_pi")
        for i in [0, 1, 2, 3, 4]:
            cron_mock().__enter__().remove.assert_any_call(i)
        for p in ["fajr", "dhuhr", "asr", "maghrib", "isha"]:
            cron_mock().__enter__().new.assert_any_call(
                command="XDG_RUNTIME_DIR=/run/user/{user_id} /opt/adhan-pi/env/bin/alert_adhan --prayer {prayer} > /dev/null 2>&1".format(
                    user_id=1000, prayer=p,
                ),
                comment="adhan_pi",
            )
    pwd_mock = get_mock("adhan_pi.cli.pwd")
    pwd_mock.getpwnam.assert_called_with("salah")
    geopy_mock = get_mock("adhan_pi.utils.Nominatim")
    geopy_mock.assert_called_with(user_agent="adhan-pi")
    geopy_mock().geocode.assert_called_with("Los Angeles")



import os

FAJR_ADHAN_MP3_PATH = os.getenv(
    "FAJR_ADHAN_MP3_PATH", "/opt/adhan-pi/static/azan-fajr.mp3"
)
ADHAN_MP3_PATH = os.getenv("ADHAN_MP3_PATH", "/opt/adhan-pi/static/azan2.mp3")
AUTOMOCK_REGISTRATION_IMPORTS = ("adhan_pi.test_mocks",)



from setuptools import find_packages, setup

######

CLASSIFIERS = [
    "Programming Language :: Python",
    "License :: OSI Approved :: MIT License",
]
DESCRIPTION = ""
KEYWORDS = ["python"]
URL = ""
AUTHOR = "Salah Ahmed"
AUTHOR_EMAIL = "salahs.email@pm.me"

REQUIREMENTS = ["attrs", "geopy", "requests", "automock"]
CRON_REQUIREMENTS = ["python-crontab", "pydub"]
TEST_REQUIREMENTS = [
    "pytest-cov",
    "pytest-freezegun",
    "coverage[toml]",
    "responses",
]
DEV_REQUIREMENTS = [
    "ipython",
    "jedi==0.17.2",
    "pdbpp",
    "black==19.10b0",
    "isort==5.6.4",
    "flake8",
    "pre-commit",
    "tox",
]

#####

with open("README.rst", encoding="utf-8") as f:
    LONG_DESCRIPTION = f.read()

setup(
    name="adhan_pi",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/x-rst",
    python_requires=">=3.6",
    classifiers=CLASSIFIERS,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    keywords=KEYWORDS,
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    include_package_data=True,
    zip_safe=False,
    install_requires=REQUIREMENTS,
    use_scm_version=True,
    setup_requires=["setuptools_scm"],
    entry_points={
        "console_scripts": [
            "schedule_prayer_cron=adhan_pi.cli:schedule_prayer_cron_runner",
            "alert_adhan=adhan_pi.cli:alert_adhan",
        ],
    },
    extras_require={
        "test": TEST_REQUIREMENTS + CRON_REQUIREMENTS,
        "dev": TEST_REQUIREMENTS + DEV_REQUIREMENTS + CRON_REQUIREMENTS,
        "cron": CRON_REQUIREMENTS,
    },
)
